/***************************************************************************
 *   Copyright (C) 2005 by Bob Dempsey                                     *
 *   bdempsey_64@msn.com                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/* This work was insipired by the work done at http://linuxk8055.free.fr/ */
/* Some of that work bears the following Copyright: */
/*
 *   Copyright (C) 2004 by Nicolas Sutre                                   *
 *   nicolas.sutre@free.fr                                                 *
*/
/*
 * The Velleman protocol was sniffed using their demo program and SniffUSB *
 * from http://benoit.papillault.free.fr/usbsnoop/
*/
/* Requires libusb from sourceforge for Linux */
/* Requires libusb-win32 from sourceforge for Windows */

#ifdef linux
#include <string.h>
#else
#include <windows.h>
#endif

#include "K8055DLL.h"
#include "usb.h"

#ifndef linux
BOOL APIENTRY DllMain( HANDLE hModule, 
                       DWORD  ul_reason_for_call, 
                       LPVOID lpReserved
					 )
{
    switch (ul_reason_for_call)
	{
		case DLL_PROCESS_ATTACH:
		case DLL_THREAD_ATTACH:
		case DLL_THREAD_DETACH:
		case DLL_PROCESS_DETACH:
			break;
    }
    return TRUE;
}
#endif

static long ana1;
static long ana2;
static long dig;
static long cnt1;
static long cnt2;
static long dbt1;
static long dbt2;

static usb_dev_handle* xsv_handle;

static void DoEmptyCmd(char cmd)
{
	char buf[8];

	memset(buf, 0, 8);

	buf[0] = cmd;

	usb_interrupt_write(xsv_handle, 0x1, buf, 8, 20);

	return;
}

static void DoDbtCmd(char cmd)
{
	char buf[8];

	memset(buf, 0, 8);

	buf[0] = cmd;
	buf[6] = (char)dbt1;
	buf[7] = (char)dbt2;

	usb_interrupt_write(xsv_handle, 0x1, buf, 8, 20);

	return;
}

static void DoCmd0()
{
	DoEmptyCmd(0);
}

static void DoCmd1(long dbt)
{
	dbt1 = dbt;
	DoDbtCmd(1);
}

static void DoCmd2(long dbt)
{
	dbt2 = dbt;
	DoDbtCmd(2);
}

static void DoCmd3()
{
	DoEmptyCmd(3);
}

static void DoCmd4()
{
	DoEmptyCmd(4);
}

static void DoCmd5(long d, long a1, long a2)
{
	char buf[8];

	memset(buf, 0, 8);

	dig = d; ana1 = a1; ana2 = a2;

	buf[0] = 0x05;
	buf[1] = (char)d;
	buf[2] = (char)a1;
	buf[3] = (char)a2;

	usb_interrupt_write(xsv_handle, 0x1, buf, 8, 20);

	return;
}

#define STR_BUFF 256

static int takeover_device(usb_dev_handle* udev, int interface)
{
	char driver_name[STR_BUFF];

	memset(driver_name, 0, STR_BUFF);

#ifdef linux
	usb_get_driver_np(udev, interface, driver_name, sizeof(driver_name));

	usb_detach_kernel_driver_np( udev, interface );
#endif

	/*claim interface */
	usb_set_configuration(udev, 1);
	if(usb_claim_interface(udev, interface) < 0)
		return -1;

	usb_set_altinterface(udev, interface);  

	return 0;
}

static usb_dev_handle *locate_xsv(int ipid)
{
	struct usb_bus *bus;
	struct usb_device *dev;
	usb_dev_handle *device_handle = NULL;

	usb_init();

	usb_find_busses();
	usb_find_devices();

	for (bus = usb_get_busses(); bus; bus = bus->next)
	{
		for (dev = bus->devices; dev; dev = dev->next)
		{

			if ( (dev->descriptor.idVendor == 0x10cf) && 
			     (dev->descriptor.idProduct == ipid ) )
			{
				if ((device_handle = usb_open(dev)) == NULL)
					return NULL;

				if (takeover_device(device_handle, 0) < 0)
					return NULL;
			}
		}
	}

	if (device_handle == 0)
		return NULL;
	else
		return device_handle;

}

K8055DLL_API long OpenDevice(long CardAddress)
{
	int ipid = 0x5500;

	switch (CardAddress)
	{
	case 0:
	default:
		ipid = 0x5500;
		break;
	case 1:
		ipid = 0x5501;
		break;
	case 2:
		ipid = 0x5502;
		break;
	case 3:
		ipid = 0x5503;
		break;
	}

	if ((xsv_handle = locate_xsv(ipid)) == NULL)
		return -1;

	DoCmd0();

	ana1 = ana2 = dig = cnt1 = cnt2 = 0;

	dbt1 = dbt2 = 4;

	return 0;
}

K8055DLL_API void CloseDevice(void)
{
	usb_close(xsv_handle);

	return;
}

K8055DLL_API long ReadAnalogChannel(long Channel)
{
	long val = 0;
	unsigned char data[8];

	usb_interrupt_read(xsv_handle, 0x81, data, 8, 20);

	if (Channel == 1)
		val = data[2];
	else
		val = data[3];

	return val;
}

K8055DLL_API void ReadAllAnalog(long* Data1, long* Data2)
{
	unsigned char data[8];

	usb_interrupt_read(xsv_handle, 0x81, data, 8, 20);

	*Data1 = data[2];
	*Data2 = data[3];

	return;
}

K8055DLL_API void OutputAnalogChannel(long Channel, long Data)
{
	long a1;
	long a2;
	long d;

	a1 = ana1;
	a2 = ana2;
	d = dig;

	if (Channel == 1)
		a1 = Data;
	else
		a2 = Data;

	DoCmd5(d, a1, a2);

	return;
}

K8055DLL_API void OutputAllAnalog(long Data1, long Data2)
{
	OutputAnalogChannel(1, Data1);
	OutputAnalogChannel(2, Data2);

	return;
}

K8055DLL_API void ClearAnalogChannel(long Channel)
{
	OutputAnalogChannel(Channel, 0);

	return;
}

K8055DLL_API void ClearAllAnalog(void)
{
	ClearAnalogChannel(1);
	ClearAnalogChannel(2);

	return;
}

K8055DLL_API void SetAnalogChannel(long Channel)
{
	OutputAnalogChannel(Channel, 255);

	return;
}

K8055DLL_API void SetAllAnalog(void)
{
	SetAnalogChannel(1);
	SetAnalogChannel(2);

	return;
}

K8055DLL_API void WriteAllDigital(long Data)
{
	long a1;
	long a2;
	long d;

	a1 = ana1;
	a2 = ana2;
	d = dig;

	d = Data;

	DoCmd5(d, a1, a2);

	return;
}

K8055DLL_API void ClearDigitalChannel(long Channel)
{
	long d;

	d = dig;

	d &= ~(1 << (Channel - 1));

	WriteAllDigital(d);

	return;
}

K8055DLL_API void ClearAllDigital(void)
{
	WriteAllDigital(0x00);

	return;
}

K8055DLL_API void SetDigitalChannel(long Channel)
{
	long d;

	d = dig;

	d |= (1 << (Channel - 1));

	WriteAllDigital(d);

	return;
}

K8055DLL_API void SetAllDigital(void)
{
	WriteAllDigital(0xff);

	return;
}

K8055DLL_API int ReadDigitalChannel(long Channel)
{
	unsigned char data[8];
	union
	{
		unsigned char data;
		struct 
		{
			unsigned sw3:1;
			unsigned b1:1;
			unsigned b2:1;
			unsigned b3:1;
			unsigned sw1:1;
			unsigned sw2:1;
			unsigned sw4:1;
			unsigned sw5:1;
		} swmask;
	} swdata;

	usb_interrupt_read(xsv_handle, 0x81, data, 8, 20);

	swdata.data = data[0];

	switch (Channel)
	{
	case 1:
		return swdata.swmask.sw1;
		break;
	case 2:
		return swdata.swmask.sw2;
		break;
	case 3:
		return swdata.swmask.sw3;
		break;
	case 4:
		return swdata.swmask.sw4;
		break;
	case 5:
		return swdata.swmask.sw5;
		break;
	default:
		break;
	}

	return 0;
}

K8055DLL_API long ReadAllDigital(void)
{
	unsigned char data[8];
	union
	{
		unsigned char data;
		struct 
		{
			unsigned sw3:1;
			unsigned b1:1;
			unsigned b2:1;
			unsigned b3:1;
			unsigned sw1:1;
			unsigned sw2:1;
			unsigned sw4:1;
			unsigned sw5:1;
		} swmask;
	} swdata;
	unsigned char out;

	usb_interrupt_read(xsv_handle, 0x81, data, 8, 20);

	swdata.data = data[0];

	out = (swdata.swmask.sw1 << 0) | (swdata.swmask.sw2 << 1) | (swdata.swmask.sw3 << 2) |
		(swdata.swmask.sw4 << 3) | (swdata.swmask.sw5 << 4);

	return out;

}

K8055DLL_API void ResetCounter(long CounterNumber)
{
	if (CounterNumber == 1)
		DoCmd3();
	else
		DoCmd4();

	return;
}

K8055DLL_API long ReadCounter(long CounterNumber)
{
	long val = 0;
	unsigned char data[8];
	unsigned short* usp;

	usb_interrupt_read(xsv_handle, 0x81, data, 8, 20);

	if (CounterNumber == 1)
	{
		usp = (unsigned short*)&data[4];
		val = *usp;
	}
	else
	{
		usp = (unsigned short*)&data[6];
		val = *usp;
	}

	return val;
}

// This is not perfect. I havn't figured out how the DebounceTime maps
// to the actual byte sent down to the k8055. The Velleman demo program
// sends down the following:
// dbt   byte
//    0     1
//    2     3
//   10     8
// 1000    88

K8055DLL_API void SetCounterDebounceTime(long CounterNr, long DebounceTime)
{
	long dbt;

	if (DebounceTime < 2)
		dbt = 1;
	else if (DebounceTime < 10)
		dbt = 3;
	else if (DebounceTime < 1000)
		dbt = 8;
	else if (DebounceTime < 5000)
		dbt = 88;
	else
		dbt = 255;

	if (CounterNr == 1)
		DoCmd1(dbt);
	else
		DoCmd2(dbt);

	return;
}
