#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>

#include <K8055DLL.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"

static int isConnected = 0;
static gint timerid = -1;

static gint timer(gpointer arg)
{
	long ana1;
	long ana2;
	long dig;
	GtkWidget* widget = arg;
	GtkProgress* in_ana;
	GtkToggleButton* inp;
	GtkEntry* cnt;
	char cnttext[80];

	// Seems we need to do this twice...
	ReadAllAnalog(&ana1, &ana2);
	ReadAllAnalog(&ana1, &ana2);

	in_ana = GTK_PROGRESS(lookup_widget(widget, "in_ana1"));
	gtk_progress_set_value(in_ana, ana1);
	in_ana = GTK_PROGRESS(lookup_widget(widget, "in_ana2"));
	gtk_progress_set_value(in_ana, ana2);

	dig = ReadAllDigital();
	inp = GTK_TOGGLE_BUTTON(lookup_widget(widget, "bt_in_dig1"));
	gtk_toggle_button_set_active(inp, (dig & (1 << 0)));
	inp = GTK_TOGGLE_BUTTON(lookup_widget(widget, "bt_in_dig2"));
	gtk_toggle_button_set_active(inp, (dig & (1 << 1)));
	inp = GTK_TOGGLE_BUTTON(lookup_widget(widget, "bt_in_dig3"));
	gtk_toggle_button_set_active(inp, (dig & (1 << 2)));
	inp = GTK_TOGGLE_BUTTON(lookup_widget(widget, "bt_in_dig4"));
	gtk_toggle_button_set_active(inp, (dig & (1 << 3)));
	inp = GTK_TOGGLE_BUTTON(lookup_widget(widget, "bt_in_dig5"));
	gtk_toggle_button_set_active(inp, (dig & (1 << 4)));

	cnt = GTK_ENTRY(lookup_widget(widget, "entry1"));
	sprintf(cnttext, "%lu", ReadCounter(1));
	gtk_entry_set_text(cnt, cnttext);
	cnt = GTK_ENTRY(lookup_widget(widget, "entry2"));
	sprintf(cnttext, "%lu", ReadCounter(2));
	gtk_entry_set_text(cnt, cnttext);

	return 1;
}

static void enable_controls(GtkButton* button)
{
	GtkObject* win;

	win = GTK_OBJECT(lookup_widget(GTK_WIDGET(button), "window1"));

#define ENABLE(ctl) gtk_widget_set_sensitive(gtk_object_get_data(win, ctl ), 1)

	ENABLE("bt_SetAllDigital");
	ENABLE("bt_ClearAllDigital");
	ENABLE("bt_SetAllAnalog");
	ENABLE("bt_ClearAllAnalog");
	ENABLE("bt_OutputTest");
	ENABLE("out_ana1");
	ENABLE("out_ana2");
	ENABLE("bt_out_dig1");
	ENABLE("bt_out_dig2");
	ENABLE("bt_out_dig3");
	ENABLE("bt_out_dig4");
	ENABLE("bt_out_dig5");
	ENABLE("bt_out_dig6");
	ENABLE("bt_out_dig7");
	ENABLE("bt_out_dig8");
	ENABLE("bt_ct1_0");
	ENABLE("bt_ct1_2");
	ENABLE("bt_ct1_10");
	ENABLE("bt_ct1_1000");
	ENABLE("bt_Reset1");
	ENABLE("bt_ct2_0");
	ENABLE("bt_ct2_2");
	ENABLE("bt_ct2_10");
	ENABLE("bt_ct2_1000");
	ENABLE("bt_Reset2");

#undef ENABLE

}


void
on_window1_destroy                     (GtkObject       *object,
                                        gpointer         user_data)
{
	if (timerid > -1) gtk_timeout_remove(timerid);

	if (isConnected) CloseDevice();

	gtk_main_quit();
}


void
on_bt_Connect_clicked                  (GtkButton       *button,
                                        gpointer         user_data)
{
	long dev = 0;
	GtkToggleButton* sk5;
	GtkToggleButton* sk6;
	gboolean isSetSK5;
	gboolean isSetSK6;
	GtkLabel* lb_Status;

	sk5 = GTK_TOGGLE_BUTTON(lookup_widget(GTK_WIDGET(button), "SK5"));
	sk6 = GTK_TOGGLE_BUTTON(lookup_widget(GTK_WIDGET(button), "SK6"));
	isSetSK5 = gtk_toggle_button_get_active(sk5);
	isSetSK6 = gtk_toggle_button_get_active(sk6);

	if (!isSetSK5 && !isSetSK6) dev = 3;
	else if (isSetSK5 && !isSetSK6) dev = 2;
	else if (!isSetSK5 && isSetSK6) dev = 1;
	else dev = 0;

	lb_Status = GTK_LABEL(lookup_widget(GTK_WIDGET(button), "lb_Status"));

	if (OpenDevice(dev) < 0)
	{
		gtk_label_set_text(lb_Status, "Card not found");
		return;
	}

	isConnected = 1;
	gtk_label_set_text(lb_Status, "Connected");

	gtk_widget_set_sensitive(GTK_WIDGET(button), 0);
	gtk_widget_set_sensitive(GTK_WIDGET(sk5), 0);
	gtk_widget_set_sensitive(GTK_WIDGET(sk6), 0);

	enable_controls(button);
}


void
on_bt_SetAllDigital_clicked            (GtkButton       *button,
                                        gpointer         user_data)
{
	SetAllDigital();

#define SETTOGGLE(ctl) gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(lookup_widget(GTK_WIDGET(button), ctl )), 1)

	SETTOGGLE("bt_out_dig1");
	SETTOGGLE("bt_out_dig2");
	SETTOGGLE("bt_out_dig3");
	SETTOGGLE("bt_out_dig4");
	SETTOGGLE("bt_out_dig5");
	SETTOGGLE("bt_out_dig6");
	SETTOGGLE("bt_out_dig7");
	SETTOGGLE("bt_out_dig8");

#undef SETTOGGLE

}


void
on_bt_ClearAllDigital_clicked          (GtkButton       *button,
                                        gpointer         user_data)
{
	ClearAllDigital();

#define CLEARTOGGLE(ctl) gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(lookup_widget(GTK_WIDGET(button), ctl )), 0)

	CLEARTOGGLE("bt_out_dig1");
	CLEARTOGGLE("bt_out_dig2");
	CLEARTOGGLE("bt_out_dig3");
	CLEARTOGGLE("bt_out_dig4");
	CLEARTOGGLE("bt_out_dig5");
	CLEARTOGGLE("bt_out_dig6");
	CLEARTOGGLE("bt_out_dig7");
	CLEARTOGGLE("bt_out_dig8");

#undef CLEARTOGGLE

}


void
on_bt_SetAllAnalog_clicked             (GtkButton       *button,
                                        gpointer         user_data)
{
	SetAllAnalog();

	gtk_adjustment_set_value(
			gtk_range_get_adjustment(
				GTK_RANGE(lookup_widget(GTK_WIDGET(button), "out_ana1"))
				),
			255.0);

	gtk_adjustment_set_value(
			gtk_range_get_adjustment(
				GTK_RANGE(lookup_widget(GTK_WIDGET(button), "out_ana2"))
				),
			255.0);
}


void
on_bt_ClearAllAnalog_clicked           (GtkButton       *button,
                                        gpointer         user_data)
{
	ClearAllAnalog();

	gtk_adjustment_set_value(
			gtk_range_get_adjustment(
				GTK_RANGE(lookup_widget(GTK_WIDGET(button), "out_ana1"))
				),
			0.0);

	gtk_adjustment_set_value(
			gtk_range_get_adjustment(
				GTK_RANGE(lookup_widget(GTK_WIDGET(button), "out_ana2"))
				),
			0.0);
}


void
on_bt_OutputTest_clicked               (GtkButton       *button,
                                        gpointer         user_data)
{
	int i;
	int j;

	ClearAllDigital();
	for (i = 1; i <= 8; i++)
	{
		SetDigitalChannel(i);
		usleep(100000);
	}
	for (i = 1; i <= 8; i++)
	{
		ClearDigitalChannel(i);
		usleep(100000);
	}
	for (i = 1; i <= 8; i++)
	{
		SetDigitalChannel(i);
		usleep(100000);
		ClearDigitalChannel(i);
	}
	ClearAllAnalog();
	for (j = 1; j <= 2; j++)
	{
		for (i = 0; i <= 255; i += 3)
			OutputAnalogChannel(j, i);
		for (i = 255; i >=0; i -= 3)
			OutputAnalogChannel(j, i);
	}
}

void
on_bt_Read_clicked                     (GtkButton       *button,
                                        gpointer         user_data)
{
	timerid = gtk_timeout_add(500, timer, button);
	gtk_widget_set_sensitive(GTK_WIDGET(button), 0);
}


void
on_bt_Reset1_clicked                   (GtkButton       *button,
                                        gpointer         user_data)
{
	ResetCounter(1);
}


void
on_bt_Reset2_clicked                   (GtkButton       *button,
                                        gpointer         user_data)
{
	ResetCounter(2);
}


void
on_bt_out_dig1_toggled                 (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
	if (gtk_toggle_button_get_active(togglebutton))
		SetDigitalChannel(1);
	else
		ClearDigitalChannel(1);
}


void
on_bt_out_dig2_toggled                 (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
	if (gtk_toggle_button_get_active(togglebutton))
		SetDigitalChannel(2);
	else
		ClearDigitalChannel(2);
}


void
on_bt_out_dig3_toggled                 (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
	if (gtk_toggle_button_get_active(togglebutton))
		SetDigitalChannel(3);
	else
		ClearDigitalChannel(3);
}


void
on_bt_out_dig4_toggled                 (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
	if (gtk_toggle_button_get_active(togglebutton))
		SetDigitalChannel(4);
	else
		ClearDigitalChannel(4);
}


void
on_bt_out_dig5_toggled                 (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
	if (gtk_toggle_button_get_active(togglebutton))
		SetDigitalChannel(5);
	else
		ClearDigitalChannel(5);
}


void
on_bt_out_dig6_toggled                 (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
	if (gtk_toggle_button_get_active(togglebutton))
		SetDigitalChannel(6);
	else
		ClearDigitalChannel(6);
}


void
on_bt_out_dig7_toggled                 (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
	if (gtk_toggle_button_get_active(togglebutton))
		SetDigitalChannel(7);
	else
		ClearDigitalChannel(7);
}


void
on_bt_out_dig8_toggled                 (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
	if (gtk_toggle_button_get_active(togglebutton))
		SetDigitalChannel(8);
	else
		ClearDigitalChannel(8);
}


void
on_bt_ct1_0_toggled                    (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
	if (gtk_toggle_button_get_active(togglebutton))
		SetCounterDebounceTime(1, 0);	
}


void
on_bt_ct1_2_toggled                    (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
	if (gtk_toggle_button_get_active(togglebutton))
		SetCounterDebounceTime(1, 2);	
}


void
on_bt_ct1_10_toggled                   (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
	if (gtk_toggle_button_get_active(togglebutton))
		SetCounterDebounceTime(1, 10);	
}


void
on_bt_ct1_1000_toggled                 (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
	if (gtk_toggle_button_get_active(togglebutton))
		SetCounterDebounceTime(1, 1000);	
}


void
on_bt_ct2_0_toggled                    (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
	if (gtk_toggle_button_get_active(togglebutton))
		SetCounterDebounceTime(2, 0);	
}


void
on_bt_ct2_2_toggled                    (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
	if (gtk_toggle_button_get_active(togglebutton))
		SetCounterDebounceTime(2, 2);	
}


void
on_bt_ct2_10_toggled                   (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
	if (gtk_toggle_button_get_active(togglebutton))
		SetCounterDebounceTime(2, 10);	
}


void
on_bt_ct2_1000_toggled                 (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
	if (gtk_toggle_button_get_active(togglebutton))
		SetCounterDebounceTime(2, 1000);	
}

